#!/usr/bin/perl

use strict;
use warnings;

use English qw(-no_match_vars);
use File::Temp;
use IPC::Run qw(run);
use Test::More;

plan tests => 4;

my $log = File::Temp->new(UNLINK => $ENV{TEST_DEBUG} ? 0 : 1);
diag("log file: $log") if $ENV{TEST_DEBUG};

my $config = File::Temp->new(UNLINK => $ENV{TEST_DEBUG} ? 0 : 1);
print {$config} <<EOF;
[setup]
templates_dir = templates
templates_theme = edugain

[federations]
edugain = t/edugain.xml

[app]
url = https://my.fqdn/accountmanager
support_email = support\@my.fqdn
name = eduGAIN Access Check

[logger]
file = $log
level = debug

[mailer]

[idp]

[database]
type = mysql
host = localhost
name = account_manager
username = user
password = password
EOF
$config->flush();
diag("test configuration: $config") if $ENV{TEST_DEBUG};

$ENV{ACCOUNTMANAGER_CONFIG} = $config;

subtest "index page" => sub {

    plan tests => 4;

    local $ENV{REQUEST_METHOD} = 'GET';
    local $ENV{QUERY_STRING}   = '';

    my ($out, $err, $rc) = run_executable('access-check-manager.cgi');
    like(
        $out,
        qr{^Content-Type: text/html; charset=utf8\r\n\r\n}m,
        'HTTP headers'
    );
    like(
        $out,
        qr{<title>eduGAIN Access Check</title>},
        'page title'
    );
    like(
        $out,
        qr{<a href="\?action=select_sp" class="button">Get started</a>},
        'start button'
    );
    is($err, '', 'empty stderr');
};

subtest "SP selection page" => sub {

    plan(skip_all => 'live database required') unless
        $ENV{TEST_DB_HOST} &&
        $ENV{TEST_DB_NAME} &&
        $ENV{TEST_DB_DRIVER} &&
        $ENV{TEST_DB_USERNAME} &&
        $ENV{TEST_DB_PASSWORD};

    plan tests => 4;

    local $ENV{REQUEST_METHOD} = 'GET';
    local $ENV{QUERY_STRING}   = 'action=select_sp';

    my ($out, $err, $rc) = run_executable('access-check-manager.cgi');
    like(
        $out,
        qr{^Content-Type: text/html; charset=utf8\r\n\r\n}m,
        'HTTP headers'
    );
    like(
        $out,
        qr{<title>eduGAIN Access Check</title>},
        'page title'
    );
    like(
        $out,
        qr{<select id="edugain" name="edugain">},
        'page content contains SP list'
    );
    is($err, '', 'empty stderr');
};

subtest "email selection page, missing entityid parameter" => sub {

    plan tests => 4;

    local $ENV{REQUEST_METHOD} = 'GET';
    local $ENV{QUERY_STRING}   = 'action=select_email&federation=edugain';

    my ($out, $err, $rc) = run_executable('access-check-manager.cgi');
    like(
        $out,
        qr{^Content-Type: text/html; charset=utf8\r\n\r\n}m,
        'HTTP headers'
    );
    like(
        $out,
        qr{<title>eduGAIN Access Check</title>},
        'page title'
    );
    like(
        $out,
        qr{Error:[\n\s]+missing parameter 'entityid'},
        'page content contains expected error message'
    );
    is($err, '', 'empty stderr');
};

subtest "email selection page, invalid entityid parameter" => sub {

    plan tests => 4;

    local $ENV{REQUEST_METHOD} = 'GET';
    local $ENV{QUERY_STRING}   = 'action=select_email&federation=edugain&entityid=foo';

    my ($out, $err, $rc) = run_executable('access-check-manager.cgi');
    like(
        $out,
        qr{^Content-Type: text/html; charset=utf8\r\n\r\n}m,
        'HTTP headers'
    );
    like(
        $out,
        qr{<title>eduGAIN Access Check</title>},
        'page title'
    );
    like(
        $out,
        qr{Error:[\n\s]+format_entityid},
        'page content contains expected error message'
    );
    is($err, '', 'empty stderr');
};

sub run_executable {
    my ($executable, $args) = @_;

    my @args = $args ? split(/\s+/, $args) : ();
    run(
        [ $EXECUTABLE_NAME, '-T', '-I', 'lib', '-I', 't', 'bin/' . $executable, @args ],
        \my ($in, $out, $err)
    );
    return ($out, $err, $CHILD_ERROR >> 8);
}
